#ifndef _COMMON_INTERFACE_H_
#define _COMMON_INTERFACE_H_

#include "stdint.h"

void Common_DataInit(void);
uint8_t Common_Get_IG_Sts(void);
uint16_t Common_Get_Act_V_Speed(void);
uint16_t Common_Get_Act_E_Speed(void);
uint16_t Common_Get_Disp_V_Speed(void);
uint16_t Common_Get_Disp_E_Speed(void);

uint8_t Common_Get_IG_Sts_Valid(void);
uint8_t Common_Get_Act_V_Speed_Valid(void);
uint8_t Common_Get_Act_E_Speed_Valid(void);
uint8_t Common_Get_Disp_V_Speed_Valid(void);
uint8_t Common_Get_Disp_E_Speed_Valid(void);


void Common_Set_IG_Sts(uint8_t Val);
void Common_Set_Act_V_Speed(uint16_t Val);
void Common_Set_Act_E_Speed(uint16_t Val);
void Common_Set_Disp_V_Speed(uint16_t Val);
void Common_Set_Disp_E_Speed(uint16_t Val);

void Common_Input_Para(void);

void Common_Set_IG_Sts_Valid(uint8_t Val);
void Common_Set_Act_V_Speed_Valid(uint8_t Val);
void Common_Set_Act_E_Speed_Valid(uint8_t Val);
void Common_Set_Disp_V_Speed_Valid(uint8_t Val);
void Common_Set_Disp_E_Speed_Valid(uint8_t Val);



uint16_t Common_Get_DispSpeed(void);


void Common_RollingCounterAdd(void);
void Common_RollingCounterClear(void);
void Common_BlockDelay(uint16_t m_MS);
uint16_t Common_ReadRollingCounter(void);


#endif
